import React from 'react'
import ReactDOM from 'react-dom/client'
import { BrowserRouter } from 'react-router-dom'
import { Auth0Provider } from '@auth0/auth0-react';
import App from './App.jsx'
import './index.css'
import './assets/designSystem.css'
import Auth0Callback from './components/Auth0Callback';
import { AuthProvider } from './contexts/AuthContext.jsx'

// Create a loading component
const LoadingFallback = () => (
  <div className="loading-container" style={{
    display: 'flex',
    justifyContent: 'center',
    alignItems: 'center',
    height: '100vh',
    flexDirection: 'column',
    backgroundColor: '#f9f9f9'
  }}>
    <div className="loading-spinner" style={{
      width: '50px',
      height: '50px',
      border: '5px solid rgba(0, 0, 0, 0.1)',
      borderTopColor: '#F7941D',
      borderRadius: '50%',
      animation: 'spin 1s linear infinite'
    }}></div>
    <p style={{ marginTop: '20px', color: '#333', fontFamily: 'Inter, sans-serif' }}>
      Loading application...
    </p>
    <style>{`
      @keyframes spin {
        to { transform: rotate(360deg); }
      }
    `}</style>
  </div>
);

// Get the base path for the application
const getBasename = () => {
  // In production when served from /onboarding/, use that as basename
  if (import.meta.env.PROD && window.location.pathname.startsWith('/onboarding')) {
    return '/onboarding';
  }
  // In development or when served from root, use no basename
  return '';
};

// Get Auth0 configuration from environment variables with defaults for development
const domain = import.meta.env.VITE_AUTH0_DOMAIN || 'dev-57jqtc1fp0y3pxzy.us.auth0.com';
const clientId = import.meta.env.VITE_AUTH0_CLIENT_ID || 'zJ10PLe0n2xCskkVebcD5xW5IjzmmgLs';
const redirectUri = import.meta.env.VITE_AUTH0_REDIRECT_URI || `${window.location.origin}/onboarding/auth-callback`;
const audience = import.meta.env.VITE_AUTH0_AUDIENCE || 'isa-onboarding-api';

console.log("Auth0 Config:", {
  domain,
  clientId,
  redirectUri,
  audience
});

// Clean up URL params if this is an Auth0 callback
const cleanUpAuthParams = () => {
  // Check if this is a callback by detecting code and state URL parameters
  const urlParams = new URLSearchParams(window.location.search);
  const isCallback = urlParams.has('code') && urlParams.has('state');
  
  if (isCallback) {
    console.log("Auth0 callback detected:", {
      code: '******', // Don't log the actual code for security
      state: urlParams.get('state')
    });
    
    // Check for error parameters
    const errorParam = urlParams.get('error');
    const errorDescriptionParam = urlParams.get('error_description');
    
    if (errorParam) {
      console.error(`Auth0 error in redirect: ${errorParam} - ${errorDescriptionParam}`);
    }
    
    // Check if this is a callback from the backend (code exchange)
    const isBackendCallback = !localStorage.getItem('auth0_callback_processed');
    
    if (isBackendCallback) {
      console.log("Detected backend callback, handling URL cleanup before Auth0Provider initialization");
      
      // Clean up the URL to remove parameters before Auth0Provider initialization
      // This prevents Auth0Provider from trying to process the same code twice
      const cleanUrl = window.location.origin + window.location.pathname;
      window.history.replaceState({}, document.title, cleanUrl);
      
      // Mark that we've processed this callback so we don't do it again
      localStorage.setItem('auth0_callback_processed', Date.now().toString());
      
      // Set a short-lived cookie as well for a backup detection method
      document.cookie = 'auth0_callback_processed=true;max-age=300;path=/;samesite=lax';
      
      console.log("URL cleaned up, ready for Auth0Provider to handle auth");
    } else {
      console.log("Direct callback from Auth0, no cleanup needed");
    }
    
    // Always update the processed timestamp
    localStorage.setItem('auth0_callback_processed', Date.now().toString());
  }
};

// Set up the redirect handler according to Auth0 documentation
const onRedirectCallback = (appState) => {
  console.log("Auth0 redirect callback triggered with appState:", appState);
  
  // After successful authentication, redirect to the app root (remove auth params)
  const basename = getBasename();
  window.history.replaceState({}, document.title, basename || '/');
};

// Clean up URL before initializing Auth0Provider
cleanUpAuthParams();

// Initialize the app with Auth0 and other providers
ReactDOM.createRoot(document.getElementById('root')).render(
  <React.StrictMode>
    <Auth0Provider
      domain={domain}
      clientId={clientId}
      authorizationParams={{
        redirect_uri: redirectUri,
        audience: audience,
        scope: 'openid profile email offline_access'
      }}
      onRedirectCallback={onRedirectCallback}
      cacheLocation="localstorage"
      useRefreshTokens={true}
    >
      <AuthProvider>
        <BrowserRouter basename={getBasename()}>
          <React.Suspense fallback={<LoadingFallback />}>
            <App />
          </React.Suspense>
        </BrowserRouter>
      </AuthProvider>
    </Auth0Provider>
  </React.StrictMode>,
)
